@@ -40,6 +40,7 @@ import pwd
 import sys
 import time
 from optparse import OptionParser
+from socket import gethostname
 
 # all of the variables below are substituted by the build system
 __VERSION__ = "unreleased_version"
@@ -72,6 +73,9 @@ def command_parse(config_opts):
     parser.add_option("--rebuild", action="store_const", const="rebuild",
                       dest="mode", default='rebuild',
                       help="rebuild the specified SRPM(s)")
+    parser.add_option("--buildsrpm", action="store_const", const="buildsrpm",
+                      dest="mode",
+                      help="Build a SRPM from spec (--spec ...) and sources (--sources ...)")
     parser.add_option("--shell", action="store_const",
                       const="shell", dest="mode",
                       help="run the specified command interactively within the chroot."
@@ -157,6 +161,11 @@ def command_parse(config_opts):
                       help="Change to the specified directory (relative to the chroot)"
                            " before running command when using --chroot")
 
+    parser.add_option("--spec", action="store",
+                    help="Specifies spec file to use to build an SRPM (used only with --buildsrpm)")
+    parser.add_option("--sources", action="store",
+                    help="Specifies sources to use to build an SRPM (used only with --buildsrpm)")
+
     # verbosity
     parser.add_option("-v", "--verbose", action="store_const", const=2,
                       dest="verbose", default=1, help="verbose build")
@@ -191,9 +200,10 @@ decorate(traceLog())
 def setup_default_config_opts(config_opts, unprivUid):
     "sets up default configuration."
     # global
-    config_opts['basedir'] = '/var/lib/mock/' # root name is automatically added to this
+    config_opts['version'] = __VERSION__
+    config_opts['basedir'] = '/var/lib/mock' # root name is automatically added to this
     config_opts['resultdir'] = '%(basedir)s/%(root)s/result'
-    config_opts['cache_topdir'] = '/var/lib/mock/cache'
+    config_opts['cache_topdir'] = '/var/cache/mock'
     config_opts['clean'] = True
     config_opts['chroothome'] = '/builddir'
     config_opts['log_config_file'] = 'logging.ini'
@@ -238,7 +248,9 @@ def setup_default_config_opts(config_opts, unprivUid):
             'root_cache_enable': True,
             'root_cache_opts': {
                 'max_age_days': 15,
-                'dir': "%(cache_topdir)s/%(root)s/root_cache/"},
+                'dir': "%(cache_topdir)s/%(root)s/root_cache/",
+                'compress_program': 'gzip',
+                'extension': '.gz'},
             'bind_mount_enable': True,
             'bind_mount_opts': {'dirs': [
                 # specify like this:
@@ -253,13 +265,12 @@ def setup_default_config_opts(config_opts, unprivUid):
     config_opts['useradd'] = \
         '/usr/sbin/useradd -o -m -u %(uid)s -g %(gid)s -d %(home)s -n %(user)s'
     config_opts['use_host_resolv'] = True
-    config_opts['chroot_setup_cmd'] = 'install buildsys-build'
+    config_opts['chroot_setup_cmd'] = 'groupinstall buildsys-build'
     config_opts['target_arch'] = 'i386'
     config_opts['rpmbuild_arch'] = None # <-- None means set automatically from target_arch
     config_opts['yum.conf'] = ''
     config_opts['more_buildreqs'] = {}
     config_opts['files'] = {}
-    config_opts['files']['etc/hosts'] = "127.0.0.1 localhost localhost.localdomain\n"
     config_opts['macros'] = {
         '%_topdir': '%s/build' % config_opts['chroothome'],
         '%_rpmfilename': '%%{NAME}-%%{VERSION}-%%{RELEASE}.%%{ARCH}.rpm',
@@ -377,6 +388,38 @@ def do_rebuild(config_opts, chroot, srpms):
             chroot.clean()
         raise
 
+def do_buildsrpm(config_opts, chroot, options, args):
+    start = time.time()
+    try:
+        # TODO: validate spec path (exists)
+        # TODO: validate SOURCES path (exists)
+
+        log.info("Start(%s)  Config(%s)" % (os.path.basename(options.spec), chroot.sharedRootName))
+        if config_opts['clean'] and chroot.state() != "clean":
+            chroot.clean()
+        chroot.init()
+
+        chroot.buildsrpm(spec=options.spec, sources=options.sources, timeout=config_opts['rpmbuild_timeout'])
+
+        elapsed = time.time() - start
+        log.info("Done(%s) Config(%s) %d minutes %d seconds"
+            % (os.path.basename(options.spec), config_opts['chroot_name'], elapsed//60, elapsed%60))
+        log.info("Results and/or logs in: %s" % chroot.resultdir)
+
+        if config_opts["cleanup_on_success"]:
+            log.info("Cleaning up build root ('clean_on_success=True')")
+            chroot.clean()
+
+    except (Exception, KeyboardInterrupt):
+        elapsed = time.time() - start
+        log.error("Exception(%s) Config(%s) %d minutes %d seconds"
+            % (os.path.basename(options.spec), chroot.sharedRootName, elapsed//60, elapsed%60))
+        log.info("Results and/or logs in: %s" % chroot.resultdir)
+        if config_opts["cleanup_on_failure"]:
+            log.info("Cleaning up build root ('clean_on_failure=True')")
+            chroot.clean()
+        raise
+
 def main(ret):
     "Main executable entry point."
     # drop unprivleged to parse args, etc.
@@ -410,7 +453,6 @@ def main(ret):
 
     uidManager = mock.uid.uidManager(unprivUid, unprivGid)
     uidManager._becomeUser(unprivUid, unprivGid)
-    del(os.environ["HOME"])
 
     # defaults
     config_opts = {}
@@ -425,9 +467,13 @@ def main(ret):
     if options.configdir:
         config_path = options.configdir
 
+    # array to save config paths
+    config_opts['config_paths'] = []
+
     # Read in the config files: default, and then user specified
     for cfg in ( os.path.join(config_path, 'site-defaults.cfg'), '%s/%s.cfg' % (config_path, options.chroot)):
         if os.path.exists(cfg):
+            config_opts['config_paths'].append(cfg)
             execfile(cfg)
         else:
             log.error("Could not find required config file: %s" % cfg)
@@ -475,6 +521,12 @@ def main(ret):
     # cmdline options override config options
     set_config_opts_per_cmdline(config_opts, options, args)
 
+    # default /etc/hosts contents
+    if not config_opts['use_host_resolv'] and not config_opts['files'].has_key('etc/hosts'):
+        config_opts['files']['etc/hosts'] = '''
+127.0.0.1 localhost localhost.localdomain
+::1       localhost localhost.localdomain localhost6 localhost6.localdomain6
+'''
     # elevate privs
     uidManager._becomeUser(0, 0)
 
@@ -494,6 +546,7 @@ def main(ret):
     ret["chroot"] = chroot
     ret["config_opts"] = config_opts
     os.umask(002)
+    os.environ["HOME"] = chroot.homedir
 
     # New namespace starting from here
     try:
@@ -515,10 +568,17 @@ def main(ret):
 
     elif options.mode == 'shell':
         chroot.tryLockBuildRoot()
+        if not os.path.exists(chroot.makeChrootPath()):
+            raise RuntimeError, "chroot %s not initialized!" % chroot.makeChrootPath()
         try:
+            chroot._setupDev()
             chroot._mountall()
             cmd = ' '.join(args)
-            status = os.system("PS1='mock-chroot> ' /usr/sbin/chroot %s %s" % (chroot.makeChrootPath(), cmd))
+            if options.unpriv:
+                arg = '--userspec=%s:%s' % (chroot.chrootuid, chroot.chrootgid)
+            else:
+                arg = ''
+            status = os.system("PS1='mock-chroot> ' /usr/sbin/chroot %s %s %s" % (arg, chroot.makeChrootPath(), cmd))
             ret['exitStatus'] = os.WEXITSTATUS(status)
 
         finally:
@@ -574,12 +634,15 @@ def main(ret):
     elif options.mode == 'rebuild':
         do_rebuild(config_opts, chroot, args)
 
+    elif options.mode == 'buildsrpm':
+        do_buildsrpm(config_opts, chroot, options, args)
+
     elif options.mode == 'orphanskill':
         mock.util.orphansKill(chroot.makeChrootPath())
     elif options.mode == 'copyin':
         chroot.tryLockBuildRoot()
         chroot._resetLogging()
-        uidManager.dropPrivsForever()
+        #uidManager.dropPrivsForever()
         if len(args) < 2:
             log.critical("Must have source and destinations for copyin")
             sys.exit(50)
@@ -633,10 +696,26 @@ if __name__ == '__main__':
     except (SystemExit,):
         raise
 
+    except (OSError,), e:
+        if e.errno == 1:
+            print
+            log.error("%s" % str(e))
+            print
+            log.error("The most common cause for this error is trying to run /usr/sbin/mock as an unprivileged user.")
+            log.error("Check your path to make sure that /usr/bin/ is listed before /usr/sbin, or manually run /usr/bin/mock to see if that fixes this problem.")
+            print
+        else:
+            raise
+
     except (KeyboardInterrupt,):
         exitStatus = 7
         log.error("Exiting on user interrupt, <CTRL>-C")
 
+    except (mock.exception.ResultDirNotAccessible,), exc:
+        exitStatus = exc.resultcode
+        log.error(str(exc))
+        killOrphans = 0
+
     except (mock.exception.BadCmdline, mock.exception.BuildRootLocked), exc:
         exitStatus = exc.resultcode
         log.error(str(exc))
