@@ -13,6 +13,13 @@ import logging
 import os
 import shutil
 import stat
+import grp
+try:
+    import uuid
+    gotuuid = True
+except:
+    gotuuid = False
+
 
 # our imports
 import mock.util
@@ -31,6 +38,7 @@ class Root(object):
         self.preExistingDeps = ""
         self.logging_initialized = False
         self.buildrootLock = None
+        self.version = config['version']
 
         self.sharedRootName = config['root']
         if config.has_key('unique-ext'):
@@ -50,6 +58,7 @@ class Root(object):
         self._state_log = getLog("mock.Root.state")
 
         # config options
+        self.configs = config['config_paths']
         self.chrootuid = config['chrootuid']
         self.chrootuser = 'mockbuild'
         self.chrootgid = config['chrootgid']
@@ -92,6 +101,9 @@ class Root(object):
         self.state("init plugins")
         self._initPlugins()
 
+        # figure out if SELinux is enabled on the host
+        self.selinux = mock.util.selinuxEnabled()
+
         # officially set state so it is logged
         self.state("start")
 
@@ -168,8 +180,9 @@ class Root(object):
         self.uidManager.dropPrivsTemp()
         try:
             mock.util.mkdirIfAbsent(self.resultdir)
-        except OSError:
-            pass
+        except (OSError,), e:
+            if e.errno == 13:
+                raise mock.exception.ResultDirNotAccessible( ResultDirNotAccessible.__doc__ % self.resultdir )
         self.uidManager.restorePrivs()
 
         # lock this buildroot so we dont get stomped on.
@@ -190,6 +203,7 @@ class Root(object):
         for item in [
                      'var/lib/rpm',
                      'var/lib/yum',
+                     'var/lib/dbus',
                      'var/log',
                      'var/lock/rpm',
                      'etc/rpm',
@@ -224,18 +238,36 @@ class Root(object):
             pass
         os.symlink('yum/yum.conf', self.makeChrootPath("etc", "yum.conf"))
 
-        # set up resolv.conf
+        # set up resolver configuration
         if self.use_host_resolv:
-            resolvdir = self.makeChrootPath('etc')
-            resolvpath = self.makeChrootPath('etc', 'resolv.conf')
-            if os.path.exists(resolvpath):
-                os.remove(resolvpath)
-            shutil.copy2('/etc/resolv.conf', resolvdir)
+            etcdir = self.makeChrootPath('etc')
 
-        # files in /etc that need doing
+            resolvconfpath = self.makeChrootPath('etc', 'resolv.conf')
+            if os.path.exists(resolvconfpath):
+                os.remove(resolvconfpath)
+            shutil.copy2('/etc/resolv.conf', etcdir)
+
+            hostspath = self.makeChrootPath('etc', 'hosts')
+            if os.path.exists(hostspath):
+                os.remove(hostspath)
+            shutil.copy2('/etc/hosts', etcdir)
+
+        if gotuuid:
+            # Anything that tries to use libdbus inside the chroot will require this
+            # FIXME - merge this code with other OS-image building code
+            machine_uuid = uuid.uuid4().hex
+            dbus_uuid_path = self.makeChrootPath('var', 'lib', 'dbus', 'machine-id')
+            f = open(dbus_uuid_path, 'w')
+            f.write(machine_uuid)
+            f.write('\n')
+            f.close()
+
+        # files that need doing
         for key in self.chroot_file_contents:
             p = self.makeChrootPath(key)
             if not os.path.exists(p):
+                # create directory if necessary
+                mock.util.mkdirIfAbsent(os.path.dirname(p))
                 # write file
                 fo = open(p, 'w+')
                 fo.write(self.chroot_file_contents[key])
@@ -248,9 +280,8 @@ class Root(object):
         self.state("running yum")
         try:
             self._mountall()
-            if not self.chrootWasCleaned:
-                self.chroot_setup_cmd = 'update'
-            self._yum(self.chroot_setup_cmd, returnOutput=1)
+            if self.chrootWasCleaned:
+                self._yum(self.chroot_setup_cmd, returnOutput=1)
         finally:
             self._umountall()
 
@@ -260,6 +291,13 @@ class Root(object):
         # create rpmbuild dir
         self._buildDirSetup()
 
+        # set up timezone to match host
+        localtimedir = self.makeChrootPath('etc')
+        localtimepath = self.makeChrootPath('etc', 'localtime')
+        if os.path.exists(localtimepath):
+            os.remove(localtimepath)
+        shutil.copy2('/etc/localtime', localtimedir)
+
         # done with init
         self._callHooks('postinit')
 
@@ -268,8 +306,9 @@ class Root(object):
         # files in /dev
         mock.util.rmtree(self.makeChrootPath("dev"))
         mock.util.mkdirIfAbsent(self.makeChrootPath("dev", "pts"))
+        mock.util.mkdirIfAbsent(self.makeChrootPath("dev", "shm"))
         prevMask = os.umask(0000)
-        devFiles = (
+        devFiles = [
             (stat.S_IFCHR | 0666, os.makedev(1, 3), "dev/null"),
             (stat.S_IFCHR | 0666, os.makedev(1, 7), "dev/full"),
             (stat.S_IFCHR | 0666, os.makedev(1, 5), "dev/zero"),
@@ -277,30 +316,48 @@ class Root(object):
             (stat.S_IFCHR | 0444, os.makedev(1, 9), "dev/urandom"),
             (stat.S_IFCHR | 0666, os.makedev(5, 0), "dev/tty"),
             (stat.S_IFCHR | 0600, os.makedev(5, 1), "dev/console"),
-            (stat.S_IFCHR | 0666, os.makedev(5, 2), "dev/ptmx"),
-        )
+        ]
+        kver = os.uname()[2]
+        # make the device node for el4 and el5
+        if kver < '2.6.19':
+            devFiles.append((stat.S_IFCHR | 0666, os.makedev(5, 2), "dev/ptmx"))
+
         for i in devFiles:
             # create node
             os.mknod( self.makeChrootPath(i[2]), i[0], i[1])
             # set context. (only necessary if host running selinux enabled.)
             # fails gracefully if chcon not installed.
-            mock.util.do(
-                ["chcon", "--reference=/%s"% i[2], self.makeChrootPath(i[2])]
-                , raiseExc=0, shell=False)
+            if self.selinux:
+                mock.util.do(
+                    ["chcon", "--reference=/%s"% i[2], self.makeChrootPath(i[2])]
+                    , raiseExc=0, shell=False)
 
         os.symlink("/proc/self/fd/0", self.makeChrootPath("dev/stdin"))
         os.symlink("/proc/self/fd/1", self.makeChrootPath("dev/stdout"))
         os.symlink("/proc/self/fd/2", self.makeChrootPath("dev/stderr"))
+
+        # symlink it for FC hosts
+        if kver >= '2.6.19':
+            os.symlink("/dev/pts/ptmx", self.makeChrootPath("dev/ptmx"))
+
         os.umask(prevMask)
 
         # mount/umount
-        umntCmd = 'umount -n %s' % self.makeChrootPath('/dev/pts')
-        if umntCmd not in self.umountCmds:
-            self.umountCmds.append(umntCmd)
+        for devUnmtCmd in (
+                'umount -n %s' % self.makeChrootPath('/dev/pts'),
+                'umount -n %s' % self.makeChrootPath('/dev/shm') ):
+            if devUnmtCmd not in self.umountCmds:
+                self.umountCmds.append(devUnmtCmd)
 
-        mntCmd = 'mount -n -t devpts mock_chroot_devpts %s' % self.makeChrootPath('/dev/pts')
-        if mntCmd not in self.mountCmds:
-            self.mountCmds.append(mntCmd)
+        mountopt = 'gid=%d,mode=0620,ptmxmode=0666' % grp.getgrnam('tty').gr_gid
+        if kver >= '2.6.29':
+            mountopt += ',newinstance'
+
+        for devMntCmd in (
+                'mount -n -t devpts -o %s mock_chroot_devpts %s' % (mountopt, self.makeChrootPath('/dev/pts')),
+                'mount -n -t tmpfs mock_chroot_shmfs %s' % self.makeChrootPath('/dev/shm') ):
+            if devMntCmd not in self.mountCmds:
+                self.mountCmds.append(devMntCmd)
 
     # bad hack
     # comment out decorator here so we dont get double exceptions in the root log
@@ -368,6 +425,7 @@ class Root(object):
         self._callHooks('earlyprebuild')
 
         try:
+            self._setupDev()
             self._mountall()
             self.uidManager.becomeUser(self.chrootuid, self.chrootgid)
             self.state("setup")
@@ -375,8 +433,6 @@ class Root(object):
             srpmChrootFilename = self._copySrpmIntoChroot(srpm)
             srpmBasename = os.path.basename(srpmChrootFilename)
 
-            # install srpm
-            os.environ["HOME"] = self.homedir
             # Completely/Permanently drop privs while running the following:
             self.doChroot(
                 ["rpm", "-Uvh", "--nodeps", srpmChrootFilename],
@@ -393,6 +449,7 @@ class Root(object):
             spec = specs[0] # if there's more than one then someone is an idiot
             chrootspec = spec.replace(self.makeChrootPath(), '') # get rid of rootdir prefix
             # Completely/Permanently drop privs while running the following:
+
             self.doChroot(
                 ["bash", "--login", "-c", 'rpmbuild -bs --target %s --nodeps %s' % (self.rpmbuild_arch, chrootspec)],
                 shell=False,
@@ -438,6 +495,64 @@ class Root(object):
             # tell caching we are done building
             self._callHooks('postbuild')
 
+
+    #
+    # UNPRIVLEGED:
+    #   Everything in this function runs as the build user
+    #       -> except hooks. :)
+    #
+    decorate(traceLog())
+    def buildsrpm(self, spec, sources, timeout):
+        """build an srpm into binary rpms, capture log"""
+
+        # tell caching we are building
+        self._callHooks('earlyprebuild')
+
+        try:
+            self._mountall()
+            self.uidManager.becomeUser(self.chrootuid, self.chrootgid)
+            self.state("setup")
+
+            # copy spec/sources
+            shutil.copy(spec, self.makeChrootPath(self.builddir, "SPECS"))
+            os.rmdir(self.makeChrootPath(self.builddir, "SOURCES"))
+            shutil.copytree(sources, self.makeChrootPath(self.builddir, "SOURCES"))
+
+            spec =  self.makeChrootPath(self.builddir, "SPECS", os.path.basename(spec))
+            chrootspec = spec.replace(self.makeChrootPath(), '') # get rid of rootdir prefix
+
+            # Completely/Permanently drop privs while running the following:
+            self.state("buildsrpm")
+            self.doChroot(
+                ["bash", "--login", "-c", 'rpmbuild -bs --target %s --nodeps %s' % (self.rpmbuild_arch, chrootspec)],
+                shell=False,
+                logger=self.build_log, timeout=timeout,
+                uid=self.chrootuid,
+                gid=self.chrootgid,
+                )
+
+            rebuiltSrpmFile = glob.glob("%s/%s/SRPMS/*.src.rpm" % (self.makeChrootPath(), self.builddir))
+            if len(rebuiltSrpmFile) == 0:
+                raise mock.exception.PkgError, "No rebuilt srpms found"
+            elif len(rebuiltSrpmFile) > 1:
+                raise mock.exception.PkgError, "Multiple rebuilt srpms found"
+
+            rebuiltSrpmFile = rebuiltSrpmFile[0]
+
+            self.root_log.debug("Copying package to result dir")
+            shutil.copy2(rebuiltSrpmFile, self.resultdir)
+
+        finally:
+            self.uidManager.restorePrivs()
+            self._umountall()
+
+            # tell caching we are done building
+            self._callHooks('postbuild')
+
+
+
+
+
     # =============
     # 'Private' API
     # =============
@@ -538,6 +653,7 @@ class Root(object):
                 fh.setFormatter(formatter)
                 fh.setLevel(logging.NOTSET)
                 log.addHandler(fh)
+                log.info("Mock Version: %s" % self.version)
         finally:
             self.uidManager.restorePrivs()
 
@@ -552,7 +668,7 @@ class Root(object):
         self.uidManager.becomeUser(self.chrootuid, self.chrootgid)
         try:
             # create dir structure
-            for subdir in [self.makeChrootPath(self.builddir, s) for s in ('RPMS', 'SRPMS', 'SOURCES', 'SPECS', 'BUILD', 'originals')]:
+            for subdir in [self.makeChrootPath(self.builddir, s) for s in ('RPMS', 'SRPMS', 'SOURCES', 'SPECS', 'BUILD', 'BUILDROOT', 'originals')]:
                 mock.util.mkdirIfAbsent(subdir)
 
             # change ownership so we can write to build home dir
